@@ -1,5 +1,6 @@
-import time
 import six
+import sys
+import time
 
 from . import exceptions
 from . import packet
@@ -33,11 +34,12 @@ class Socket(object):
             raise exceptions.QueueEmpty()
         if packets == [None]:
             return []
-        try:
-            packets.append(self.queue.get(block=False))
-            self.queue.task_done()
-        except self.server._async['queue'].Empty:
-            pass
+        while True:
+            try:
+                packets.append(self.queue.get(block=False))
+                self.queue.task_done()
+            except self.server._async['queue'].Empty:
+                break
         return packets
 
     def receive(self, pkt):
@@ -53,7 +55,7 @@ class Socket(object):
             self.send(packet.Packet(packet.PONG, pkt.data))
         elif pkt.packet_type == packet.MESSAGE:
             self.server._trigger_event('message', self.sid, pkt.data,
-                                       async=self.server.async_handlers)
+                                       run_async=self.server.async_handlers)
         elif pkt.packet_type == packet.UPGRADE:
             self.send(packet.Packet(packet.NOOP))
         elif pkt.packet_type == packet.CLOSE:
@@ -61,14 +63,26 @@ class Socket(object):
         else:
             raise exceptions.UnknownPacketError()
 
-    def send(self, pkt):
-        """Send a packet to the client."""
+    def check_ping_timeout(self):
+        """Make sure the client is still sending pings.
+
+        This helps detect disconnections for long-polling clients.
+        """
         if self.closed:
-            raise IOError('Socket is closed')
+            raise exceptions.SocketIsClosedError()
         if time.time() - self.last_ping > self.server.ping_timeout:
             self.server.logger.info('%s: Client is gone, closing socket',
                                     self.sid)
-            self.close(wait=False, abort=True)
+            # Passing abort=False here will cause close() to write a
+            # CLOSE packet. This has the effect of updating half-open sockets
+            # to their correct state of disconnected
+            self.close(wait=False, abort=False)
+            return False
+        return True
+
+    def send(self, pkt):
+        """Send a packet to the client."""
+        if not self.check_ping_timeout():
             return
         self.queue.put(pkt)
         self.server.logger.info('%s: Sending packet %s data %s',
@@ -89,9 +103,10 @@ class Socket(object):
                                                           start_response)
         try:
             packets = self.poll()
-        except exceptions.QueueEmpty as e:
+        except exceptions.QueueEmpty:
+            exc = sys.exc_info()
             self.close(wait=False)
-            raise e
+            six.reraise(*exc)
         return packets
 
     def handle_post_request(self, environ):
@@ -109,18 +124,12 @@ class Socket(object):
         """Close the socket connection."""
         if not self.closed and not self.closing:
             self.closing = True
-            reraise_exc = None
-            try:
-                self.server._trigger_event('disconnect', self.sid, async=False)
-            except Exception as e:
-                reraise_exc = e
+            self.server._trigger_event('disconnect', self.sid, run_async=False)
             if not abort:
                 self.send(packet.Packet(packet.CLOSE))
             self.closed = True
             if wait:
                 self.queue.join()
-            if reraise_exc:
-                raise reraise_exc
 
     def _upgrade_websocket(self, environ, start_response):
         """Upgrade the connection from polling to websocket."""
@@ -193,7 +202,6 @@ class Socket(object):
         self.server.logger.info(
             '%s: Upgrade to websocket successful', self.sid)
 
-        reraise_exc = None
         while True:
             p = None
             try:
@@ -214,19 +222,19 @@ class Socket(object):
             pkt = packet.Packet(encoded_packet=p)
             try:
                 self.receive(pkt)
-            except exceptions.UnknownPacketError:
+            except exceptions.UnknownPacketError:  # pragma: no cover
                 pass
-            except Exception as e:
-                # if we get an unexpected exception (such as something in an
-                # application event handler) we close the connection properly
-                # and then reraise the exception
-                reraise_exc = e
+            except exceptions.SocketIsClosedError:  # pragma: no cover
+                self.server.logger.info('Receive error -- socket is closed')
+                break
+            except:  # pragma: no cover
+                # if we get an unexpected exception we log the error and exit
+                # the connection properly
+                self.server.logger.exception('Unknown receive error')
                 break
 
         self.queue.put(None)  # unlock the writer task so that it can exit
         writer_task.join()
-        self.close(wait=True, abort=True)
-        if reraise_exc:
-            raise reraise_exc
+        self.close(wait=False, abort=True)
 
         return []
